from typing import Dict

from bobocep.rules.events.bobo_event import BoboEvent


class PrimitiveEvent(BoboEvent):
    """A primitive event.

    :param timestamp: The event timestamp indicating when it was first
                      generated.
    :type timestamp: int

    :param data: The event data, defaults to an empty dict.
    :type data: Dict[str, str], optional

    :param event_id: The event ID, defaults to a randomly generated ID.
    :type event_id: str, optional
    """

    def __init__(self,
                 timestamp: int,
                 data: Dict[str, str] = None,
                 event_id: str = None) -> None:
        super().__init__(timestamp=timestamp,
                         data=data,
                         event_id=event_id)

    def to_dict(self) -> dict:
        """
        :return: A dict representation of the object.
        """

        return {
            self.TIMESTAMP: self.timestamp,
            self.DATA: self.data,
            self.ID: self.id
        }



from abc import ABC, abstractmethod

from bobocep.rules.events.histories.bobo_history import BoboHistory


class IDeciderSubscriber(ABC):
    """An interface to subscribe to Decider events."""

    @abstractmethod
    def on_decider_complex_event(self,
                                 nfa_name: str,
                                 history: BoboHistory) -> None:
        """
        When a Decider identifies a complex event in the event stream.

        :param nfa_name: The complex event name.
        :type nfa_name: str

        :param history: The events that triggered the complex event.
        :type history: BoboHistory
        """



from time import time_ns
from typing import Dict

from bobocep.rules.actions.bobo_action import BoboAction
from bobocep.rules.events.composite_event import CompositeEvent


class RateLimitAction(BoboAction):
    """
    Ensures that CompositeEvent instances of a given name are only produced
    at a given rate i.e. one event with name *n* in *t* seconds.
    If multiple instances are created within the rate, the subsequence
    ones are dropped i.e. the action returns :code:`False`.

    :param pairs: Key-value pairs, where the key is the CompositeEvent name
                  and the value is the desired rate, in seconds.
    :type pairs: Dict[str, int]

    :param rate_other: The rate for all other events that are not specifically
                       named in pairs, defaults to 0.
    :type rate_other: int, optional
    """

    def __init__(self,
                 pairs: Dict[str, int],
                 rate_other: int = 0) -> None:
        super().__init__()

        self._pairs = pairs
        self._rate_other = rate_other
        self._names = self._pairs.keys()
        self._last = {}

    def perform_action(self, event: CompositeEvent) -> bool:
        name = event.name
        rate = self._pairs[name] if name in self._names else self._rate_other

        if rate > 0:
            if name not in self._last:
                self._last[name] = 0

            return (time_ns() - self._last[name]) / 1e-9 >= rate

        return True
